@@ -14,6 +14,7 @@ from mkdocs.config import config_options
 from mkdocs.plugins import BasePlugin
 from md2cf.confluence_renderer import ConfluenceRenderer
 from os import environ
+from pathlib import Path
 
 TEMPLATE_BODY = "<p> TEMPLATE </p>"
 
@@ -51,6 +52,8 @@ class MkdocsWithConfluence(BasePlugin):
         self.confluence_mistune = mistune.Markdown(renderer=self.confluence_renderer)
         self.simple_log = False
         self.flen = 1
+        self.session = requests.Session()
+        self.page_attachments = {}
 
     def on_nav(self, nav, config, files):
         MkdocsWithConfluence.tab_nav = []
@@ -143,8 +146,7 @@ class MkdocsWithConfluence(BasePlugin):
 
     def on_page_markdown(self, markdown, page, config, files):
         MkdocsWithConfluence._id += 1
-        self.pw = self.config["password"]
-        self.user = self.config["username"]
+        self.session.auth = (self.config["username"], self.config["password"])
 
         if self.enabled:
             if self.simple_log is True:
@@ -218,10 +220,14 @@ class MkdocsWithConfluence(BasePlugin):
                         if self.config["debug"]:
                             print(f"DEBUG    - FOUND IMAGE: {match.group(1)}")
                         attachments.append(match.group(1))
-                    for match in re.finditer(r"!\[[\w\. -]*\]\((?!http|file)(.*)\)", markdown):
+                    for match in re.finditer(r"!\[[\w\. -]*\]\((?!http|file)([^\s,]*).*\)", markdown):
+                        file_path = match.group(1).lstrip("./\\")
+                        attachments.append(file_path)
+
                         if self.config["debug"]:
-                            print(f"DEBUG    - FOUND IMAGE: {match.group(1)}")
-                        attachments.append("docs/" + match.group(1))
+                            print(f"DEBUG    - FOUND IMAGE: {file_path}")
+                        attachments.append("docs/" + file_path.replace("../", ""))
+
                 except AttributeError as e:
                     if self.config["debug"]:
                         print(f"DEBUG    - WARN(({e}): No images found in markdown. Proceed..")
@@ -268,7 +274,6 @@ class MkdocsWithConfluence(BasePlugin):
                     self.update_page(page.title, confluence_body)
                     for i in MkdocsWithConfluence.tab_nav:
                         if page.title in i:
-                            n_kol = len(i + " *NEW PAGE*")
                             print(f"INFO    - Mkdocs With Confluence: {i} *UPDATE*")
                 else:
                     if self.config["debug"]:
@@ -297,7 +302,6 @@ class MkdocsWithConfluence(BasePlugin):
                             self.add_page(parent1, main_parent_id, body)
                             for i in MkdocsWithConfluence.tab_nav:
                                 if parent1 in i:
-                                    n_kol = len(i + "INFO    - Mkdocs With Confluence:" + " *NEW PAGE*")
                                     print(f"INFO    - Mkdocs With Confluence: {i} *NEW PAGE*")
                             time.sleep(1)
 
@@ -310,12 +314,9 @@ class MkdocsWithConfluence(BasePlugin):
                         self.add_page(parent, second_parent_id, body)
                         for i in MkdocsWithConfluence.tab_nav:
                             if parent in i:
-                                n_kol = len(i + "INFO    - Mkdocs With Confluence:" + " *NEW PAGE*")
                                 print(f"INFO    - Mkdocs With Confluence: {i} *NEW PAGE*")
                         time.sleep(1)
 
-                    # if self.config['debug']:
-
                     if parent_id is None:
                         for i in range(11):
                             while parent_id is None:
@@ -335,17 +336,10 @@ class MkdocsWithConfluence(BasePlugin):
                     print(f"Trying to ADD page '{page.title}' to parent0({parent}) ID: {parent_id}")
                     for i in MkdocsWithConfluence.tab_nav:
                         if page.title in i:
-                            n_kol = len(i + "INFO    - Mkdocs With Confluence:" + " *NEW PAGE*")
                             print(f"INFO    - Mkdocs With Confluence: {i} *NEW PAGE*")
 
                 if attachments:
-                    if self.config["debug"]:
-                        print(f"\nDEBUG    - UPLOADING ATTACHMENTS TO CONFLUENCE, DETAILS:\n" f"FILES: {attachments}\n")
-
-                    n_kol = len("  *NEW ATTACHMENTS({len(attachments)})*")
-                    print(f"\033[A\033[F\033[{n_kol}G  *NEW ATTACHMENTS({len(attachments)})*")
-                    for f in attachments:
-                        self.add_or_update_attachment(page.title, f)
+                    self.page_attachments[page.title] = attachments
 
             except IndexError as e:
                 if self.config["debug"]:
@@ -354,6 +348,20 @@ class MkdocsWithConfluence(BasePlugin):
 
         return markdown
 
+    def on_post_page(self, output, page, config):
+        site_dir = config.get("site_dir")
+        attachments = self.page_attachments.get(page.title, [])
+
+        if self.config["debug"]:
+            print(f"\nDEBUG    - UPLOADING ATTACHMENTS TO CONFLUENCE FOR {page.title}, DETAILS:")
+            print(f"FILES: {attachments}  \n")
+        for attachment in attachments:
+            if self.config["debug"]:
+                print(f"DEBUG    - looking for {attachment} in {site_dir}")
+            for p in Path(site_dir).rglob(f"*{attachment}"):
+                self.add_or_update_attachment(page.title, p)
+        return output
+
     def on_page_content(self, html, page, config, files):
         return html
 
@@ -428,9 +436,8 @@ class MkdocsWithConfluence(BasePlugin):
         headers = {"X-Atlassian-Token": "no-check"}  # no content-type here!
         if self.config["debug"]:
             print(f"URL: {url}")
-        auth = (self.user, self.pw)
 
-        r = requests.get(url, headers=headers, params={"filename": name, "expand": "version"}, auth=auth)
+        r = self.session.get(url, headers=headers, params={"filename": name, "expand": "version"})
         r.raise_for_status()
         with nostdout():
             response_json = r.json()
@@ -443,19 +450,20 @@ class MkdocsWithConfluence(BasePlugin):
 
         url = self.config["host_url"] + "/" + page_id + "/child/attachment/" + existing_attachment["id"] + "/data"
         headers = {"X-Atlassian-Token": "no-check"}  # no content-type here!
+
         if self.config["debug"]:
             print(f"URL: {url}")
-        filename = filepath
-        auth = (self.user, self.pw)
+
+        filename = os.path.basename(filepath)
 
         # determine content-type
-        content_type, encoding = mimetypes.guess_type(filename)
+        content_type, encoding = mimetypes.guess_type(filepath)
         if content_type is None:
             content_type = "multipart/form-data"
-        files = {"file": (filename, open(filename, "rb"), content_type), "comment": message}
+        files = {"file": (filename, open(Path(filepath), "rb"), content_type), "comment": message}
 
         if not self.dryrun:
-            r = requests.post(url, headers=headers, files=files, auth=auth)
+            r = self.session.post(url, headers=headers, files=files)
             r.raise_for_status()
             print(r.json())
             if r.status_code == 200:
@@ -469,19 +477,19 @@ class MkdocsWithConfluence(BasePlugin):
 
         url = self.config["host_url"] + "/" + page_id + "/child/attachment"
         headers = {"X-Atlassian-Token": "no-check"}  # no content-type here!
+
         if self.config["debug"]:
             print(f"URL: {url}")
-        filename = filepath
-        auth = (self.user, self.pw)
+
+        filename = os.path.basename(filepath)
 
         # determine content-type
-        content_type, encoding = mimetypes.guess_type(filename)
+        content_type, encoding = mimetypes.guess_type(filepath)
         if content_type is None:
             content_type = "multipart/form-data"
-        files = {"file": (filename, open(filename, "rb"), content_type), "comment": message}
-
+        files = {"file": (filename, open(filepath, "rb"), content_type), "comment": message}
         if not self.dryrun:
-            r = requests.post(url, headers=headers, files=files, auth=auth)
+            r = self.session.post(url, headers=headers, files=files)
             print(r.json())
             r.raise_for_status()
             if r.status_code == 200:
@@ -496,8 +504,7 @@ class MkdocsWithConfluence(BasePlugin):
         url = self.config["host_url"] + "?title=" + name_confl + "&spaceKey=" + self.config["space"] + "&expand=history"
         if self.config["debug"]:
             print(f"URL: {url}")
-        auth = (self.user, self.pw)
-        r = requests.get(url, auth=auth)
+        r = self.session.get(url)
         r.raise_for_status()
         with nostdout():
             response_json = r.json()
@@ -519,7 +526,6 @@ class MkdocsWithConfluence(BasePlugin):
         if self.config["debug"]:
             print(f"URL: {url}")
         headers = {"Content-Type": "application/json"}
-        auth = (self.user, self.pw)
         space = self.config["space"]
         data = {
             "type": "page",
@@ -531,7 +537,7 @@ class MkdocsWithConfluence(BasePlugin):
         if self.config["debug"]:
             print(f"DATA: {data}")
         if not self.dryrun:
-            r = requests.post(url, json=data, headers=headers, auth=auth)
+            r = self.session.post(url, json=data, headers=headers)
             r.raise_for_status()
             if r.status_code == 200:
                 if self.config["debug"]:
@@ -552,7 +558,6 @@ class MkdocsWithConfluence(BasePlugin):
             if self.config["debug"]:
                 print(f"URL: {url}")
             headers = {"Content-Type": "application/json"}
-            auth = (self.user, self.pw)
             space = self.config["space"]
             data = {
                 "id": page_id,
@@ -564,7 +569,7 @@ class MkdocsWithConfluence(BasePlugin):
             }
 
             if not self.dryrun:
-                r = requests.put(url, json=data, headers=headers, auth=auth)
+                r = self.session.put(url, json=data, headers=headers)
                 r.raise_for_status()
                 if r.status_code == 200:
                     if self.config["debug"]:
@@ -581,8 +586,7 @@ class MkdocsWithConfluence(BasePlugin):
             print(f"INFO    -   * Mkdocs With Confluence: Find PAGE VERSION, PAGE NAME: {page_name}")
         name_confl = page_name.replace(" ", "+")
         url = self.config["host_url"] + "?title=" + name_confl + "&spaceKey=" + self.config["space"] + "&expand=version"
-        auth = (self.user, self.pw)
-        r = requests.get(url, auth=auth)
+        r = self.session.get(url)
         r.raise_for_status()
         with nostdout():
             response_json = r.json()
@@ -601,8 +605,7 @@ class MkdocsWithConfluence(BasePlugin):
         idp = self.find_page_id(name)
         url = self.config["host_url"] + "/" + idp + "?expand=ancestors"
 
-        auth = (self.user, self.pw)
-        r = requests.get(url, auth=auth)
+        r = self.session.get(url)
         r.raise_for_status()
         with nostdout():
             response_json = r.json()
